@@ -7,27 +7,34 @@
 @contact : mmmaaaggg@163.com
 @desc    :
 """
-import os
-import time
 import functools
-from datetime import datetime, date, timedelta
-import pytz
-import numpy as np
-import re
-import pandas as pd
-from collections import OrderedDict
-import logging
-import warnings
-from functools import reduce
-import xlrd
-import threading
-import math
-import platform
-from matplotlib.font_manager import FontManager
-import subprocess
 import importlib
-import shutil
+import json
+import logging
+import math
+import os
+import platform
 import random
+import re
+import shutil
+import subprocess
+import threading
+import time
+import typing
+import warnings
+from enum import Enum
+from collections import OrderedDict
+from datetime import datetime, date, timedelta
+from functools import reduce
+from io import StringIO
+from typing import ClassVar, Iterator, Optional, List
+
+import numpy as np
+import pandas as pd
+import pytz
+import simplejson
+import xlrd
+from matplotlib.font_manager import FontManager
 from numba import njit
 
 logger = logging.getLogger(__name__)
@@ -43,8 +50,20 @@ PATTERN_DATETIME_FORMAT_RESTRICT = re.compile(r"\d{4}(\D)*\d{2}(\D)*\d{2} \d{2}(
 PATTERN_DATETIME_FORMAT = re.compile(r"\d{4}(\D)*\d{1,2}(\D)*\d{1,2} \d{1,2}(\D)*\d{1,2}(\D)*\d{1,2}")
 
 
+def thread_save(func):
+    """线程安全装饰器，用于该函数执行过程线程安全"""
+    lock = threading.Lock()
+
+    def wrapper(*args, **kwargs):
+        with lock:
+            func(*args, **kwargs)
+
+    return wrapper
+
+
 def active_coroutine(func):
     """装饰器：向前执行第一个 yield 表达式，预激活 func"""
+
     @functools.wraps(func)
     def primer(*arg, **kwargs):
         gen = func(*arg, **kwargs)
@@ -122,6 +141,36 @@ def try_2_float(data):
         return None
 
 
+def csv_split_chunk(file_name, csv_split_chunk_size=1024 * 1024 * 8) -> Iterator[StringIO]:
+    """按照尺寸将csv文件切割成为N组字符流"""
+    with open(file_name) as f:
+        first_line = f.readline()
+        line = f.readline()
+        stream = None
+        while line != "":
+            # 初始化 steam
+            if stream is None:
+                stream = StringIO()
+                stream.writelines([first_line, line])
+                cur_size = len(first_line) + len(line)
+            else:
+                # 填充数据到 steam
+                stream.write(line)
+
+            cur_size += len(line)
+            if cur_size >= csv_split_chunk_size:
+                # 返回 steam
+                stream.seek(0)
+                yield stream
+                stream = None
+
+            line = f.readline()
+
+        if stream is not None:
+            stream.seek(0)
+            yield stream
+
+
 def split_chunk(l: list, n: int):
     """
     将数组按照给定长度进行分割
@@ -185,6 +234,9 @@ def populate_obj(model_obj, data_dic: dict, attr_list=None, error_if_no_key=Fals
 
 
 def log_param_when_exception(func):
+    """
+    当函数异常是 log 记录异常
+    """
 
     @functools.wraps(func)
     def handler(*arg, **kwargs):
@@ -253,7 +305,7 @@ def try_n_times(times=3, sleep_time=3, logger: logging.Logger = None, exception=
     :param timeout: 超时时间
     :return:
     """
-    last_invoked_time = [None]
+    last_invoked_time: List[Optional[float]] = [None]
 
     def wrap_func(func):
 
@@ -342,7 +394,7 @@ def time_2_str(t):
 
 def date_time_2_str(d, t):
     """将日期与时间组合成  '%Y-%m-%d %H:%M:%S' 字符串 """
-    return date_2_str(d) +  ' ' + time_2_str(t)
+    return date_2_str(d) + ' ' + time_2_str(t)
 
 
 def str_2_datetime(datetime_str, format=STR_FORMAT_DATETIME):
@@ -700,7 +752,7 @@ def is_pattern_type(obj):
         return isinstance(obj, re._pattern_type)
 
 
-def get_folder_path(target_folder_name=None, create_if_not_found=True):
+def get_folder_path(target_folder_name=None, create_if_not_found=True) -> Optional[str]:
     """
     获得系统缓存目录路径
     :param target_folder_name: 缓存目录名称 或 正则表达式
@@ -1534,23 +1586,17 @@ def load_class(module_name, class_name):
     return class_meta
 
 
-def decorator_timer(func):
-    """
-    为当期程序进行计时
-    :param func:
-    :return:
-    """
-    @functools.wraps(func)
-    def timer_func(*args, **kwargs):
-        start = time.time()
-        try:
-            return func(*args, **kwargs)
-        finally:
-            end = time.time()
-            estimate = time.strftime('%H:%M:%S', time.gmtime(end - start))
-            logger.info('%s 运行时间：%s 相关参数 (%s, %s)', func.__name__, estimate, args, kwargs)
-
-    return timer_func
+def is_sub_class(sub_cls: ClassVar, cls: ClassVar) -> bool:
+    """检查某一个类是否是另一个类的子类"""
+    is_ok = False
+    try:
+        for _ in sub_cls.mro():
+            if _ == cls:
+                is_ok = True
+                break
+    except AttributeError:
+        pass
+    return is_ok
 
 
 def open_file_with_system_app(file_path, asyn=True):
@@ -1673,7 +1719,7 @@ def copy_module_file_to(module_str_or_class, folder_path):
     elif isinstance(module_str_or_class, type):
         module = importlib.import_module(module_str_or_class.__module__)
     elif hasattr(module_str_or_class, '__file__'):
-        module =module_str_or_class
+        module = module_str_or_class
     else:
         raise ValueError(f'{module_str_or_class} <{type(module_str_or_class)}> 不是有效的对象')
 
@@ -1762,6 +1808,69 @@ def _test_weighted_sample():
     print(result)
 
 
+class JsonEncoder(json.JSONEncoder):
+    """Json编码器"""
+
+    def default(self, obj):
+        if isinstance(obj, np.int64) or isinstance(obj, np.int32):
+            return int(obj)
+        elif isinstance(obj, np.float64) or isinstance(obj, np.float32) or isinstance(obj, np.float16):
+            return float(obj)
+        elif isinstance(obj, datetime):
+            return datetime_2_str(obj)
+        elif isinstance(obj, date):
+            return date_2_str(obj)
+        elif isinstance(obj, Enum):
+            return obj.name
+
+
+class SimpleJsonEncoder(simplejson.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, np.int64) or isinstance(obj, np.int32):
+            return int(obj)
+        elif isinstance(obj, np.float64) or isinstance(obj, np.float32) or isinstance(obj, np.float16):
+            return float(obj)
+        elif isinstance(obj, datetime):
+            return datetime_2_str(obj)
+        elif isinstance(obj, date):
+            return date_2_str(obj)
+        elif isinstance(obj, Enum):
+            return obj.name
+
+
+def dict_2_jsonable(dic: typing.Dict, **kwargs) -> typing.Dict:
+    """将字典对象中的各项转换为可Json的对象"""
+    kwargs.setdefault('ignore_nan', True)
+    kwargs.setdefault('cls', SimpleJsonEncoder)
+    return json.loads(obj_2_json(dic, **kwargs))
+
+
+def obj_2_json(dic: typing.Union[typing.Dict, typing.List], **kwargs) -> str:
+    """将字典/列表对象转换为Json字符串"""
+    kwargs.setdefault('ignore_nan', True)
+    kwargs.setdefault('cls', SimpleJsonEncoder)
+    return simplejson.dumps(dic, **kwargs)
+
+
+def _test_dict_2_jsonable():
+    class EnumTest(Enum):
+        T1 = 't1'
+        T2 = 't2'
+
+    obj = dict(
+        dt=datetime.now(),
+        date=date.today(),
+        float64=np.float64(2.34),
+        nan=np.nan,
+        enum=EnumTest.T2
+    )
+    ret = dict_2_jsonable(obj)
+    print(ret)
+    expect = {'dt': datetime_2_str(obj['dt']), 'date': date_2_str(obj['date']), 'float64': 2.34, 'nan': None,
+              'enum': 'T2'}
+    assert all([k1 == k2 and v1 == v2 for (k1, v1), (k2, v2) in zip(ret.items(), expect.items())])
+
+
 if __name__ == "__main__":
     pass
     # logging.basicConfig(level=logging.DEBUG,
@@ -1868,4 +1977,5 @@ if __name__ == "__main__":
     # _test_get_module_file_path()
     # _test_copy_module_file_to()
     # _test_copy_folder_to()
-    _test_get_idx_nb()
+    # _test_get_idx_nb()
+    _test_dict_2_jsonable()
